"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
function methodDelegateOnInterface(interfaceDeclaration, property) {
    //TODO: we do only first level - by configuration we could do it recursively 
    const propertySignature = typeof property === 'string' ? interfaceDeclaration.getProperty(property) : property;
    const decl = propertySignature.getType().getSymbol().getDeclarations()[0];
    if (ts_simple_ast_1.TypeGuards.isClassDeclaration(decl)) {
        const methods = getClassMethodStructures(decl, propertySignature.getName());
        interfaceDeclaration.addMethods(methods);
    }
    if (ts_simple_ast_1.TypeGuards.isInterfaceDeclaration(decl)) {
        interfaceDeclaration.addMethods(getInterfaceMethodStructures(decl));
    }
}
exports.methodDelegateOnInterface = methodDelegateOnInterface;
function methodDelegateOnClass(classDeclaration, property) {
    //TODO: we do only first level - by configuration we could do it recursively 
    const propertyDeclaration = typeof property === 'string' ? classDeclaration.getProperty(property) : property;
    const decl = propertyDeclaration.getType().getSymbol().getDeclarations()[0];
    if (ts_simple_ast_1.TypeGuards.isClassDeclaration(decl) || ts_simple_ast_1.TypeGuards.isInterfaceDeclaration(decl)) {
        classDeclaration.addMethods(getClassMethodStructures(decl, propertyDeclaration.getName()));
    }
    else {
        // TODO: what do we do here ? 
    }
}
exports.methodDelegateOnClass = methodDelegateOnClass;
function getInterfaceMethodStructures(decl) {
    return decl.getMethods().map(method => ({
        name: method.getName(),
        hasQuestionToken: method.hasQuestionToken(),
        returnType: method.getReturnTypeNodeOrThrow().getText(),
        docs: method.getJsDocs().map(d => ({ description: d.getInnerText().replace(/\r?\n/g, "\r\n") })),
        typeParameters: method.getTypeParameters().map(p => ({
            name: p.getName(),
            constraint: p.getConstraintNode() == null ? undefined : p.getConstraintNode().getText()
        })),
        parameters: method.getParameters().map(p => ({
            name: p.getNameOrThrow(),
            hasQuestionToken: p.hasQuestionToken(),
            type: p.getTypeNodeOrThrow().getText()
        }))
    }));
}
function getClassMethodStructures(decl, memberName) {
    if (ts_simple_ast_1.TypeGuards.isClassDeclaration(decl)) {
        return decl.getInstanceMethods().map(method => ({
            name: method.getName(),
            returnType: method.getReturnTypeNode() == null ? undefined : method.getReturnTypeNodeOrThrow().getText(),
            docs: method.getJsDocs().map(d => ({ description: d.getInnerText().replace(/\r?\n/g, "\r\n") })),
            scope: method.hasScopeKeyword() ? method.getScope() : undefined,
            typeParameters: method.getTypeParameters().map(p => ({
                name: p.getName(),
                constraint: p.getConstraintNode() == null ? undefined : p.getConstraintNode().getText()
            })),
            parameters: method.getParameters().map(p => ({
                name: p.getNameOrThrow(),
                hasQuestionToken: p.hasQuestionToken(),
                type: p.getTypeNode() == null ? undefined : p.getTypeNodeOrThrow().getText(),
                isRestParameter: p.isRestParameter(),
                scope: p.hasScopeKeyword() ? p.getScope() : undefined
            })),
            bodyText: `var a = 1; return Math.random() as any` //this.${memberName}(${method.getParameters().map(p=>p.getName()).join(', ')}); `
        }));
    }
    else if (ts_simple_ast_1.TypeGuards.isInterfaceDeclaration(decl)) {
        return decl.getMethods().map(method => ({
            name: method.getName(),
            returnType: method.getReturnTypeNode() == null ? undefined : method.getReturnTypeNodeOrThrow().getText(),
            docs: method.getJsDocs().map(d => ({ description: d.getInnerText().replace(/\r?\n/g, "\r\n") })),
            scope: ts_simple_ast_1.Scope.Public,
            typeParameters: method.getTypeParameters().map(p => ({
                name: p.getName(),
                constraint: p.getConstraintNode() == null ? undefined : p.getConstraintNode().getText()
            })),
            parameters: method.getParameters().map(p => ({
                name: p.getNameOrThrow(),
                hasQuestionToken: p.hasQuestionToken(),
                type: p.getTypeNode() == null ? undefined : p.getTypeNodeOrThrow().getText(),
                isRestParameter: p.isRestParameter(),
                scope: p.hasScopeKeyword() ? p.getScope() : undefined
            })),
            bodyText: `return this.${memberName}.${method.getName()}(${method.getParameters().map(p => p.getName()).join(', ')}); `
        }));
    }
    // const methods: MethodDeclaration[]|MethodSignature[] = TypeGuards.isClassDeclaration(decl) ? decl.getInstanceMethods() : TypeGuards.isInterfaceDeclaration(decl) ? decl.getMethods() : []
}
exports.getClassMethodStructures = getClassMethodStructures;
//# sourceMappingURL=methodDelegate.js.map